__all__ = ['StringSplitMethod',
           'StrLenMethod',
           ]

from boa3.internal.model.builtin.native.stdlib.stringsplitmethod import StringSplitMethod
from boa3.internal.model.builtin.native.stdlib.strlenmethod import StrLenMethod
