from boa3.internal.model.builtin.interop.storage.put.storageputboolmethod import StoragePutBoolMethod
from boa3.internal.model.builtin.interop.storage.put.storageputbytesmethod import StoragePutBytesMethod
from boa3.internal.model.builtin.interop.storage.put.storageputdictmethod import StoragePutDictMethod
from boa3.internal.model.builtin.interop.storage.put.storageputecpointmethod import StoragePutECPointMethod
from boa3.internal.model.builtin.interop.storage.put.storageputintmethod import StoragePutIntMethod
from boa3.internal.model.builtin.interop.storage.put.storageputlistmethod import StoragePutListMethod
from boa3.internal.model.builtin.interop.storage.put.storageputobjectmethod import StoragePutObjectMethod
from boa3.internal.model.builtin.interop.storage.put.storageputstrmethod import StoragePutStrMethod
from boa3.internal.model.builtin.interop.storage.put.storageputuint160method import StoragePutUInt160Method
from boa3.internal.model.builtin.interop.storage.put.storageputuint256method import StoragePutUInt256Method
