from boa3.internal.model.builtin.interop.storage.getcheck.storagegetboolmethod import StorageTryGetBoolMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetbytesmethod import StorageTryGetBytesMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetdictmethod import StorageTryGetDictMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetecpointmethod import StorageTryGetECPointMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetintmethod import StorageTryGetIntMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetlistmethod import StorageTryGetListMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetobjectmethod import StorageTryGetObjectMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetstrmethod import StorageTryGetStrMethod
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetuint160method import StorageTryGetUInt160Method
from boa3.internal.model.builtin.interop.storage.getcheck.storagegetuint256method import StorageTryGetUInt256Method
