from boa3.internal.model.builtin.interop.storage.get.storagegetboolmethod import StorageGetBoolMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetbytesmethod import StorageGetBytesMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetdictmethod import StorageGetDictMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetecpointmethod import StorageGetECPointMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetintmethod import StorageGetIntMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetlistmethod import StorageGetListMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetobjectmethod import StorageGetObjectMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetstrmethod import StorageGetStrMethod
from boa3.internal.model.builtin.interop.storage.get.storagegetuint160method import StorageGetUInt160Method
from boa3.internal.model.builtin.interop.storage.get.storagegetuint256method import StorageGetUInt256Method
